@@ -41,6 +41,7 @@ public:
 
 protected:
 	virtual CommMessageOut *message_out();
+	virtual int keep_alive_timeout();
 	virtual bool init_success();
 	virtual bool finish_once();
 
@@ -61,7 +62,8 @@ CommMessageOut *ComplexDnsTask::message_out()
 	DnsResponse *resp = this->get_resp();
 	TransportType type = this->get_transport_type();
 
-	/* Set these field every time, in case of reconstruct on redirect */
+	if (req->get_id() == 0)
+		req->set_id(this->get_task_seq() + 1);
 	resp->set_request_id(req->get_id());
 	resp->set_request_name(req->get_question_name());
 	req->set_single_packet(type == TT_UDP);
@@ -70,6 +72,14 @@ CommMessageOut *ComplexDnsTask::message_out()
 	return this->WFClientTask::message_out();
 }
 
+int ComplexDnsTask::keep_alive_timeout()
+{
+	if (this->get_task_seq() == (1 << 16) - 2)
+		return 0;
+
+	return this->WFClientTask::keep_alive_timeout();
+}
+
 bool ComplexDnsTask::init_success()
 {
 	if (uri_.scheme && strcasecmp(uri_.scheme, "dnss") == 0)
