@@ -101,6 +101,9 @@ using WFEmptyTask = WFGenericTask;
 using WFDynamicTask = WFGenericTask;
 using dynamic_create_t = std::function<SubTask *(WFDynamicTask *)>;
 
+using repeated_create_t = std::function<SubTask *(WFRepeaterTask *)>;
+using repeater_callback_t = std::function<void (WFRepeaterTask *)>;
+
 using module_callback_t = std::function<void (const WFModuleTask *)>;
 
 class WFTaskFactory
@@ -290,6 +293,20 @@ public:
 										 const std::string& queue_name,
 										 FUNC&& func, ARGS&&... args);
 
+	/* Create 'Go' task on user's executor and execution queue. */
+	template<class FUNC, class... ARGS>
+	static WFGoTask *create_go_task(ExecQueue *queue, Executor *executor,
+									FUNC&& func, ARGS&&... args);
+
+	template<class FUNC, class... ARGS>
+	static WFGoTask *create_timedgo_task(time_t seconds, long nanoseconds,
+										 ExecQueue *queue, Executor *executor,
+										 FUNC&& func, ARGS&&... args);
+
+	/* For capturing 'task' itself in go task's running function. */
+	template<class FUNC, class... ARGS>
+	static void reset_go_task(WFGoTask *task, FUNC&& func, ARGS&&... args);
+
 public:
 	static WFGraphTask *create_graph_task(graph_callback_t callback)
 	{
@@ -304,6 +321,12 @@ public:
 
 	static WFDynamicTask *create_dynamic_task(dynamic_create_t create);
 
+	static WFRepeaterTask *create_repeater_task(repeated_create_t create,
+												repeater_callback_t callback)
+	{
+		return new WFRepeaterTask(std::move(create), std::move(callback));
+	}
+
 public:
 	static WFModuleTask *create_module_task(SubTask *first,
 											module_callback_t callback)
@@ -343,6 +366,12 @@ public:
 								 int retry_max,
 								 std::function<void (T *)> callback);
 
+	static T *create_client_task(TransportType type,
+								 const struct sockaddr *addr,
+								 socklen_t addrlen,
+								 int retry_max,
+								 std::function<void (T *)> callback);
+
 public:
 	static T *create_server_task(CommService *service,
 								 std::function<void (T *)>& process);
@@ -353,22 +382,39 @@ class WFThreadTaskFactory
 {
 private:
 	using T = WFThreadTask<INPUT, OUTPUT>;
-	using MT = WFMultiThreadTask<INPUT, OUTPUT>;
 
 public:
 	static T *create_thread_task(const std::string& queue_name,
 								 std::function<void (INPUT *, OUTPUT *)> routine,
 								 std::function<void (T *)> callback);
 
+	/* Create thread task with running time limit. */
+	static T *create_thread_task(time_t seconds, long nanoseconds,
+								 const std::string& queue_name,
+								 std::function<void (INPUT *, OUTPUT *)> routine,
+								 std::function<void (T *)> callback);
+
+public:
+	/* Create thread task on user's executor and execution queue. */
+	static T *create_thread_task(ExecQueue *queue, Executor *executor,
+								 std::function<void (INPUT *, OUTPUT *)> routine,
+								 std::function<void (T *)> callback);
+
+	/* With running time limit. */
+	static T *create_thread_task(time_t seconds, long nanoseconds,
+								 ExecQueue *queue, Executor *executor,
+								 std::function<void (INPUT *, OUTPUT *)> routine,
+								 std::function<void (T *)> callback);
+
+private:
+	using MT = WFMultiThreadTask<INPUT, OUTPUT>;
+
+public:
 	static MT *create_multi_thread_task(const std::string& queue_name,
 										std::function<void (INPUT *, OUTPUT *)> routine,
 										size_t nthreads,
 										std::function<void (MT *)> callback);
 
-public:
-	static T *create_thread_task(ExecQueue *queue, Executor *executor,
-								 std::function<void (INPUT *, OUTPUT *)> routine,
-								 std::function<void (T *)> callback);
 };
 
 #include "WFTaskFactory.inl"
