@@ -325,6 +325,7 @@ public:
 	static __CommManager *get_instance()
 	{
 		static __CommManager kInstance;
+		__CommManager::created_ = true;
 		return &kInstance;
 	}
 
@@ -353,6 +354,8 @@ public:
 		return fio_service_;
 	}
 
+	static bool is_created() { return created_; }
+
 private:
 	__CommManager():
 		fio_service_(NULL),
@@ -382,8 +385,13 @@ private:
 	__FileIOService *fio_service_;
 	volatile bool fio_flag_;
 	std::mutex fio_mutex_;
+
+private:
+	static bool created_;
 };
 
+bool __CommManager::created_ = false;
+
 class __ExecManager
 {
 protected:
@@ -628,6 +636,11 @@ DnsCache WFGlobal::dns_cache_;
 WFDnsResolver WFGlobal::dns_resolver_;
 WFNameService WFGlobal::name_service_(&WFGlobal::dns_resolver_);
 
+bool WFGlobal::is_scheduler_created()
+{
+	return __CommManager::is_created();
+}
+
 CommScheduler *WFGlobal::get_scheduler()
 {
 	return __CommManager::get_instance()->get_scheduler();
@@ -689,14 +702,22 @@ void WFGlobal::register_scheme_port(const std::string& scheme,
 	__WFGlobal::get_instance()->register_scheme_port(scheme, port);
 }
 
-void WFGlobal::sync_operation_begin()
+int WFGlobal::sync_operation_begin()
 {
-	__WFGlobal::get_instance()->sync_operation_begin();
+	if (WFGlobal::is_scheduler_created() &&
+		WFGlobal::get_scheduler()->is_handler_thread())
+	{
+		__WFGlobal::get_instance()->sync_operation_begin();
+		return 1;
+	}
+
+	return 0;
 }
 
-void WFGlobal::sync_operation_end()
+void WFGlobal::sync_operation_end(int cookie)
 {
-	__WFGlobal::get_instance()->sync_operation_end();
+	if (cookie)
+		__WFGlobal::get_instance()->sync_operation_end();
 }
 
 static inline const char *__get_ssl_error_string(int error)
