@@ -13,7 +13,8 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  Authors: Wu Jiaxu (wujiaxu@sogou-inc.com)
+  Authors: Li Yingxin (liyingxin@sogou-inc.com)
+           Wang Zhulei (wangzhulei@sogou-inc.com)
 */
 
 #include <pthread.h>
@@ -56,10 +57,6 @@ UPSAddrParams::UPSAddrParams(const struct AddressParams *params,
 							 const std::string& address) :
 	PolicyAddrParams(params)
 {
-	static std::hash<std::string> std_hash;
-	for (int i = 0; i < VIRTUAL_GROUP_SIZE; i++)
-		this->consistent_hash[i] = std_hash(address + "|v" + std::to_string(i));
-
 	this->weight = params->weight;
 	this->server_type = params->server_type;
 	this->group_id = params->group_id;
@@ -168,7 +165,7 @@ bool UPSGroupPolicy::select(const ParsedURI& uri, WFNSTracing *tracing,
 
 	this->check_breaker();
 
-	// select_addr == NULL will only happened in consistent_hash
+	// select_addr == NULL will happen only in consistent_hash
 	EndpointAddress *select_addr = this->first_strategy(uri, tracing);
 
 	if (!select_addr || select_addr->fail_count >= select_addr->params->max_fails)
@@ -194,17 +191,17 @@ bool UPSGroupPolicy::select(const ParsedURI& uri, WFNSTracing *tracing,
 }
 
 /*
- * flag true : return an available one. If not exists, return NULL.
- *      false: means addr maybe group-alive.
- *      	   If addr is not available, get one from addr->group.
+ * addr_failed true: return an available one. If not exists, return NULL.
+ * 			  false: means addr maybe group-alive.
+ * 					 If addr is not available, get one from addr->group.
  */
 EndpointAddress *UPSGroupPolicy::check_and_get(EndpointAddress *addr,
-											   bool flag,
+											   bool addr_failed,
 											   WFNSTracing *tracing)
 {
 	UPSAddrParams *params = static_cast<UPSAddrParams *>(addr->params);
 
-	if (flag == true) // && addr->fail_count >= addr->params->max_fails
+	if (addr_failed) // means fail_count >= max_fails
 	{
 		if (params->group_id == -1)
 			return NULL;
@@ -333,9 +330,6 @@ void UPSGroupPolicy::add_server_locked(EndpointAddress *addr)
 	else
 		group->backups.push_back(addr);
 	pthread_mutex_unlock(&group->mutex);
-	this->server_list_change(addr, ADD_SERVER);
-
-	return;
 }
 
 int UPSGroupPolicy::remove_server_locked(const std::string& address)
@@ -380,7 +374,6 @@ int UPSGroupPolicy::remove_server_locked(const std::string& address)
 				}
 			}
 
-			this->server_list_change(addr, REMOVE_SERVER);
 			if (--addr->ref == 0)
 			{
 				this->pre_delete_server(addr);
@@ -397,37 +390,87 @@ int UPSGroupPolicy::remove_server_locked(const std::string& address)
 	return ret;
 }
 
-EndpointAddress *UPSGroupPolicy::consistent_hash_with_group(unsigned int hash)
+EndpointAddress *UPSGroupPolicy::consistent_hash_with_group(unsigned int hash,
+															WFNSTracing *tracing)
 {
-	const UPSAddrParams *params;
-	EndpointAddress *addr = NULL;
-	unsigned int min_dis = (unsigned int)-1;
+	if (this->nalives == 0)
+		return NULL;
 
-	for (EndpointAddress *server : this->servers)
+	std::map<unsigned int, EndpointAddress *>::iterator it;
+	it = this->addr_hash.lower_bound(hash);
+
+	if (it == this->addr_hash.end())
+		it = this->addr_hash.begin();
+
+	while (!this->is_alive(it->second))
 	{
-		if (this->is_alive(server))
+		it++;
+		if (it == this->addr_hash.end())
+			it = this->addr_hash.begin();
+	}
+
+	return this->check_and_get(it->second, false, tracing);
+}
+
+void UPSGroupPolicy::hash_map_add_addr(EndpointAddress *addr)
+{
+	UPSAddrParams *params = static_cast<UPSAddrParams *>(addr->params);
+
+	if (params->server_type == 0)
+	{
+		static std::hash<std::string> std_hash;
+		unsigned int hash_value;
+		size_t ip_count = this->server_map[addr->address].size();
+
+		for (int i = 0; i < VIRTUAL_GROUP_SIZE * params->weight; i++)
 		{
-			params = static_cast<UPSAddrParams *>(server->params);
+			hash_value = std_hash(addr->address + "|v" + std::to_string(i) +
+								  "|n" + std::to_string(ip_count));
+			this->addr_hash.insert(std::make_pair(hash_value, addr));
+		}
+	}
+}
 
-			for (int i = 0; i < VIRTUAL_GROUP_SIZE; i++)
-			{
-				unsigned int dis = std::min<unsigned int>
-								   (hash - params->consistent_hash[i],
-								   params->consistent_hash[i] - hash);
+void UPSGroupPolicy::hash_map_remove_addr(const std::string& address)
+{
+	std::map<unsigned int, EndpointAddress *>::iterator it;
 
-				if (dis < min_dis)
-				{
-					min_dis = dis;
-					addr = server;
-				}
-			}
+	for (it = this->addr_hash.begin(); it != this->addr_hash.end();)
+	{
+		if (it->second->address == address)
+			this->addr_hash.erase(it++);
+		else
+			it++;
+	}
+}
+
+int UPSRoundRobinPolicy::remove_server_locked(const std::string& address)
+{
+	if (servers.size() != 0)
+	{
+		size_t cur_idx = this->cur_idx % servers.size();
+
+		for (size_t i = 0; i < cur_idx; i++)
+		{
+			if (this->servers[i]->address == address)
+				this->cur_idx--;
 		}
 	}
 
-	if (!addr)
-		return NULL;
+	return UPSGroupPolicy::remove_server_locked(address);
+}
 
-	return this->check_and_get(addr, false, NULL);
+EndpointAddress *UPSRoundRobinPolicy::first_strategy(const ParsedURI& uri,
+													 WFNSTracing *tracing)
+{
+	return this->servers[this->cur_idx++ % this->servers.size()];
+}
+
+EndpointAddress *UPSRoundRobinPolicy::another_strategy(const ParsedURI& uri,
+													   WFNSTracing *tracing)
+{
+	EndpointAddress *addr = this->servers[this->cur_idx++ % this->servers.size()];
+	return this->check_and_get(addr, false, tracing);
 }
 
 void UPSWeightedRandomPolicy::add_server_locked(EndpointAddress *addr)
@@ -437,7 +480,6 @@ void UPSWeightedRandomPolicy::add_server_locked(EndpointAddress *addr)
 	UPSGroupPolicy::add_server_locked(addr);
 	if (params->server_type == 0)
 		this->total_weight += params->weight;
-	return;
 }
 
 int UPSWeightedRandomPolicy::remove_server_locked(const std::string& address)
@@ -561,31 +603,24 @@ void UPSWeightedRandomPolicy::fuse_one_server(const EndpointAddress *addr)
 EndpointAddress *UPSVNSWRRPolicy::first_strategy(const ParsedURI& uri,
 												 WFNSTracing *tracing)
 {
-	int idx = this->cur_idx;
-	for (int i = 0; i < this->total_weight; i++)
+	int idx = this->cur_idx.fetch_add(1);
+	int pos = 0;
+	for (int i = 0; i < this->total_weight; i++, idx++)
 	{
-		if (this->cur_idx >= this->pre_generated_vec.size() &&
-			(int)this->pre_generated_vec.size() < this->total_weight)
-		{
-			this->init_virtual_nodes();
-		}
-
-		idx = (this->cur_idx + i) % this->pre_generated_vec.size();
-		int pos = this->pre_generated_vec[idx];
+		pos = this->pre_generated_vec[idx % this->pre_generated_vec.size()];
 		if (WFServiceGovernance::in_select_history(tracing, this->servers[pos]))
 			continue;
 
 		break;
 	}
-	this->cur_idx = idx + 1;
-	return this->servers[idx];
+	return this->servers[pos];
 }
 
 void UPSVNSWRRPolicy::init_virtual_nodes()
 {
 	UPSAddrParams *params;
 	size_t start_pos = this->pre_generated_vec.size();
-	size_t end_pos = std::min(this->total_weight - start_pos, this->servers.size()) + start_pos;
+	size_t end_pos = this->total_weight;
 	this->pre_generated_vec.resize(end_pos);
 
 	for (size_t i = start_pos; i < end_pos; i++)
@@ -619,7 +654,6 @@ void UPSVNSWRRPolicy::add_server_locked(EndpointAddress *addr)
 {
 	UPSWeightedRandomPolicy::add_server_locked(addr);
 	init();
-	return;
 }
 
 int UPSVNSWRRPolicy::remove_server_locked(const std::string& address)
@@ -636,7 +670,20 @@ EndpointAddress *UPSConsistentHashPolicy::first_strategy(const ParsedURI& uri,
 										uri.path ? uri.path : "",
 										uri.query ? uri.query : "",
 										uri.fragment ? uri.fragment : "");
-	return this->consistent_hash_with_group(hash_value);
+	return this->consistent_hash_with_group(hash_value, tracing);
+}
+
+void UPSConsistentHashPolicy::add_server_locked(EndpointAddress *addr)
+{
+	UPSGroupPolicy::add_server_locked(addr);
+	this->hash_map_add_addr(addr);
+}
+
+int UPSConsistentHashPolicy::remove_server_locked(const std::string& address)
+{
+	this->hash_map_remove_addr(address);
+
+	return UPSGroupPolicy::remove_server_locked(address);
 }
 
 EndpointAddress *UPSManualPolicy::first_strategy(const ParsedURI& uri,
@@ -659,6 +706,22 @@ EndpointAddress *UPSManualPolicy::another_strategy(const ParsedURI& uri,
 										uri.path ? uri.path : "",
 										uri.query ? uri.query : "",
 										uri.fragment ? uri.fragment : "");
-	return this->consistent_hash_with_group(hash_value);
+	return this->consistent_hash_with_group(hash_value, tracing);
+}
+
+void UPSManualPolicy::add_server_locked(EndpointAddress *addr)
+{
+	UPSGroupPolicy::add_server_locked(addr);
+
+	if (this->try_another)
+		this->hash_map_add_addr(addr);
+}
+
+int UPSManualPolicy::remove_server_locked(const std::string& address)
+{
+	if (this->try_another)
+		this->hash_map_remove_addr(address);
+
+	return UPSGroupPolicy::remove_server_locked(address);
 }
 
