@@ -107,11 +107,14 @@ public:
 
 	KafkaList& operator= (const KafkaList& copy)
 	{
-		this->~KafkaList();
-		this->ref = copy.ref;
-		++*this->ref;
-		this->t_list = copy.t_list;
-		this->curpos = copy.curpos;
+		if (this != &copy)
+		{
+			this->~KafkaList();
+			this->ref = copy.ref;
+			++*this->ref;
+			this->t_list = copy.t_list;
+			this->curpos = copy.curpos;
+		}
 		return *this;
 	}
 
@@ -227,10 +230,13 @@ public:
 
 	KafkaMap& operator= (const KafkaMap& copy)
 	{
-		this->~KafkaMap();
-		this->ref = copy.ref;
-		++*this->ref;
-		this->t_map = copy.t_map;
+		if (this != &copy)
+		{
+			this->~KafkaMap();
+			this->ref = copy.ref;
+			++*this->ref;
+			this->t_map = copy.t_map;
+		}
 		return *this;
 	}
 
@@ -619,12 +625,23 @@ public:
 		return *this;
 	}
 
-	KafkaConfig& operator= (const KafkaConfig& copy)
+	KafkaConfig(const KafkaConfig& copy)
 	{
-		this->~KafkaConfig();
 		this->ptr = copy.ptr;
 		this->ref = copy.ref;
 		++*this->ref;
+	}
+
+	KafkaConfig& operator= (const KafkaConfig& copy)
+	{
+		if (this != &copy)
+		{
+			this->~KafkaConfig();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
+
 		return *this;
 	}
 
@@ -633,7 +650,6 @@ public:
 private:
 	kafka_config_t *ptr;
 	std::atomic<int> *ref;
-	std::string sasl_buf;
 };
 
 class KafkaRecord
@@ -723,12 +739,23 @@ public:
 		return *this;
 	}
 
-	KafkaRecord& operator= (KafkaRecord& copy)
+	KafkaRecord(const KafkaRecord& copy)
 	{
-		this->~KafkaRecord();
 		this->ptr = copy.ptr;
 		this->ref = copy.ref;
 		++*this->ref;
+	}
+
+	KafkaRecord& operator= (const KafkaRecord& copy)
+	{
+		if (this != &copy)
+		{
+			this->~KafkaRecord();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
+
 		return *this;
 	}
 
@@ -793,10 +820,18 @@ public:
 	void set_offset_timestamp(long long tm) { this->ptr->offset_timestamp = tm; }
 
 	long long get_high_watermark() const { return this->ptr->high_watermark; }
+	void set_high_watermark(long long offset) const { this->ptr->high_watermark = offset; }
 
 	long long get_low_watermark() const { return this->ptr->low_watermark; }
 	void set_low_watermark(long long offset) { this->ptr->low_watermark = offset; }
 
+	void clear_records()
+	{
+		INIT_LIST_HEAD(&this->ptr->record_list);
+		this->curpos = &this->ptr->record_list;
+		this->startpos = this->endpos = this->curpos;
+	}
+
 public:
 	KafkaToppar()
 	{
@@ -847,15 +882,19 @@ public:
 		this->endpos = copy.endpos;
 	}
 
-	KafkaToppar& operator= (KafkaToppar& copy)
+	KafkaToppar& operator= (const KafkaToppar& copy)
 	{
-		this->~KafkaToppar();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
-		this->curpos = copy.curpos;
-		this->startpos = copy.startpos;
-		this->endpos = copy.endpos;
+		if (this != &copy)
+		{
+			this->~KafkaToppar();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+			this->curpos = copy.curpos;
+			this->startpos = copy.startpos;
+			this->endpos = copy.endpos;
+		}
+
 		return *this;
 	}
 
@@ -933,6 +972,19 @@ public:
 		this->curpos = this->curpos->prev;
 	}
 
+	KafkaRecord *get_tail_record()
+	{
+		if (&this->ptr->record_list != this->ptr->record_list.prev)
+		{
+			return (KafkaRecord *)list_entry(this->ptr->record_list.prev,
+					KafkaRecord, list);
+		}
+		else
+		{
+			return NULL;
+		}
+	}
+
 private:
 	struct list_head list;
 	kafka_topic_partition_t *ptr;
@@ -1037,11 +1089,14 @@ public:
 
 	KafkaBroker& operator= (const KafkaBroker& copy)
 	{
-		this->~KafkaBroker();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		if (this->ref)
-			++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaBroker();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			if (this->ref)
+				++*this->ref;
+		}
 
 		return *this;
 	}
@@ -1196,12 +1251,16 @@ public:
 		++*this->ref;
 	}
 
-	KafkaMeta& operator= (KafkaMeta& copy)
+	KafkaMeta& operator= (const KafkaMeta& copy)
 	{
-		this->~KafkaMeta();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaMeta();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
+
 		return *this;
 	}
 
@@ -1252,6 +1311,44 @@ private:
 	friend const KafkaMeta *get_meta(const char *topic, KafkaMetaList *meta_list);
 };
 
+class KafkaMetaSubscriber
+{
+public:
+	void set_meta(KafkaMeta *meta)
+	{
+		this->meta = meta;
+	}
+
+	const KafkaMeta *get_meta() const
+	{
+		return this->meta;
+	}
+
+	void add_member(kafka_member_t *member)
+	{
+		this->member_vec.push_back(member);
+	}
+
+	const std::vector<kafka_member_t *> *get_member() const
+	{
+		return &this->member_vec;
+	}
+
+	static bool cmp(const kafka_member_t *m1, const kafka_member_t *m2)
+	{
+		return strcmp(m1->member_id, m2->member_id) < 0;
+	}
+
+	void sort_by_member()
+	{
+		std::sort(this->member_vec.begin(), this->member_vec.end(), cmp);
+	}
+
+private:
+	KafkaMeta *meta;
+	std::vector<kafka_member_t *> member_vec;
+};
+
 class KafkaCgroup
 {
 public:
@@ -1352,6 +1449,9 @@ public:
 
 	int run_assignor(KafkaMetaList *meta_list, const char *protocol_name);
 
+	void add_subscriber(KafkaMetaList *meta_list,
+						std::vector<KafkaMetaSubscriber> *subscribers);
+
 	static int kafka_range_assignor(kafka_member_t **members,
 									int member_elements,
 									void *meta_topic);
@@ -1463,44 +1563,6 @@ private:
 	friend class KafkaList<KafkaBlock>;
 };
 
-class KafkaMetaSubscriber
-{
-public:
-	void set_meta(KafkaMeta *meta)
-	{
-		this->meta = meta;
-	}
-
-	const KafkaMeta *get_meta() const
-	{
-		return this->meta;
-	}
-
-	void add_member(kafka_member_t *member)
-	{
-		this->member_vec.push_back(member);
-	}
-
-	const std::vector<kafka_member_t *> *get_member() const
-	{
-		return &this->member_vec;
-	}
-
-	static bool cmp(const kafka_member_t *m1, const kafka_member_t *m2)
-	{
-		return strcmp(m1->member_id, m2->member_id) < 0;
-	}
-
-	void sort_by_member()
-	{
-		std::sort(this->member_vec.begin(), this->member_vec.end(), cmp);
-	}
-
-private:
-	KafkaMeta *meta;
-	std::vector<kafka_member_t *> member_vec;
-};
-
 class KafkaBuffer
 {
 public:
